/*============================================================================
  File:     iFTSOverview.sql

  Summary:  This script illustrates some of the new fulltext search 
			functionality in SQL Server 2008. 
			
			NOTE: You MUST run this script with SQLCMD mode enabled in SSMS
			or use SQLCMD utility to run it. To enable SQLCMD mode in SSMS,
			choose Query/SQLCMD mode from the SSMS main menu.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  For more scripts and sample code, check out http://www.SQLskills.com

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

:on error exit
go
-- this is to enable filestream at an engine level
-- NOTE: Filestream must also be configured at an OS level using 
-- SQL Server Configuration Manager (see BOL for details)
EXEC sp_configure 'filestream access level', '2'
RECONFIGURE
GO

!! if not exist c:\Files mkdir C:\Files
!! if exist c:\Files\LogFiles rmdir /s /q C:\Files\LogFiles
go
use master
go
if exists (select 1 from sys.databases where name = 'iFTS')
	alter database iFTS set single_user with rollback immediate
go
if exists (select 1 from sys.databases where name = 'iFTS')
	drop database iFTS 
go
create database iFTS ON 
( NAME = iFTS_data, 
    FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\iFTS_data.mdf',
    SIZE = 5MB,
    MAXSIZE = 50MB, 
    FILEGROWTH = 15%),
 FILEGROUP FileStreamGroup1 CONTAINS FILESTREAM
  ( NAME = FileStreamLogFiles, 
    FILENAME = N'C:\Files\LogFiles')
go
use iFTS
go
if object_id('iFTSDemo') IS NOT NULL 
	drop table iFTSDemo
go
create table iFTSDemo(filename varchar(2000))
go
go
!! dir  /b /S c:\*.log > AllLogFiles.txt
go
--!! type AllLogFIles.txt
go
!! bcp iFTS.dbo.iFTSDemo in C:\AllLogFiles.txt -S. -T -c -t@
go
alter table iFTSDemo add fileId int identity(1,1)
                       , blob [varbinary](max) FILESTREAM NULL
                       , blobType varchar(10) null
                       , guidKey [UNIQUEIDENTIFIER]  NOT NULL ROWGUIDCOL DEFAULT NEWID()
                       , constraint UQ_iFTSDemo_guidKey UNIQUE (guidKey)
                       , constraint UQ_iFTSDemo_fileId UNIQUE (fileId)
go
:on error continue
go
set nocount on
declare @i int = 0

while @i < (select count(1) from iFTSDemo)
  begin
  declare @filename varchar(200) = (select filename from iFTSDemo where fileId = @i)
  declare @sql nvarchar(max) = '
  update iFTSDemo
     set blob = ( SELECT d
       FROM OPENROWSET(BULK N''' + replace(@filename,'''','''''') + ''', SINGLE_BLOB) AS Document(d)) 
        ,blobType = ''log''
   where fileId = @fileid'
 
  exec sp_executesql @sql, N'@fileId int',@i
 
  set @i += 1
  end
go
create fulltext catalog FTCatalog 
go
--drop FULLTEXT INDEX ON [dbo].[iFTSDemo]
go
--Create new index with new STOPLIST option
CREATE FULLTEXT INDEX ON [dbo].[iFTSDemo](
[blob] TYPE COLUMN blobtype)
KEY INDEX [UQ_iFTSDemo_GuidKey]ON ([FTCatalog], FILEGROUP [PRIMARY])
WITH (CHANGE_TRACKING = AUTO, STOPLIST = SYSTEM)
go
--wait until the data has been indexed
--if this doesn't complete check the crawler log
--C:\Program Files\Microsoft SQL Server\MSSQL10.MSSQLSERVER\MSSQL\LOG\SQLFT********.Log
while (select objectpropertyex(object_id('iftsdemo'),'TableFulltextPopulateStatus'))=1
  waitfor delay '00:00:05'
  
print 'indexing completed'
go

--Show the query plan highlight the docmap lookup
-- cost .01277
-- cost .0032
select * from containstable(iftsdemo,*, 'error')
go

drop FULLTEXT INDEX ON [dbo].[iFTSDemo]
go

CREATE FULLTEXT INDEX ON [dbo].[iFTSDemo](
[blob] TYPE COLUMN blobtype)
KEY INDEX UQ_iFTSDemo_fileId ON ([FTCatalog], FILEGROUP [PRIMARY])
WITH (CHANGE_TRACKING = AUTO, STOPLIST = SYSTEM)
go

while (select objectpropertyex(object_id('iftsdemo'),'TableFulltextPopulateStatus'))=1
  waitfor delay '00:00:05'
  
print 'indexing completed'
go

--Show the new query plan without the docmap lookup
select * from containstable(iftsdemo,*, 'error')
go

--Show the new dmvs that display the index contents
--Can be used to understand the content or the data
--Index keywords
select * from sys.dm_fts_index_keywords(db_id(), object_id('iFtsDemo'))
order by document_count desc
go

--Document keywords
--This dmv returns ALL keywords for your index and will ve VERY LARGE
--Because we are using an integer key the document_id is our table key
--
--If you want to use there in your application schedule a process to perisist this data to a table
select d.* 
from sys.dm_fts_index_keywords_by_document( DB_ID(), OBJECT_ID('iFtsDemo') ) d
join iFtsDemo on iFtsDemo.fileId = d.document_id
--where filename = 'c:\windows\svcpack.log'
order by occurrence_count desc

go
--If we were using a non integer key then we could map the documentId to key using the docidmap table. 
--
select * from sys.fulltext_indexes
select * from sys.internal_tables

go
select * FROM sys.dm_fts_parser (' "The Microsoft business analysis" ', 1033, 0, 0)
go
select * FROM sys.dm_fts_parser (' "multi-million" ', 1033, 0, 0)
go
exec sys.sp_fulltext_load_thesaurus_file 1033
go
select * FROM sys.dm_fts_parser (' "bill gates" and "multi-millionaire" ', 1033, 0, 0)
go
select * FROM sys.dm_fts_parser ('FORMSOF( THESAURUS,"bill gates multi-millionaire" )', 1033, 0, 0)
go
select * FROM sys.dm_fts_parser ('FORMSOF( THESAURUS,"bill gate microsft" )', 1033, 0, 0)
go
go


